#!/opt/imh-python/bin/python3
import subprocess
import argparse
from pathlib import Path
from rads.color import red, green


def main():
    parser = argparse.ArgumentParser(
        description="Arguements for user, domain, and rule",
        epilog="Ex.  modsec_disable  --user userna5 --domain domain.tld --rule 90010",
    )
    parser.add_argument("-u", "--user", type=str, help="Cpanel user this is for")
    parser.add_argument(
        "-d",
        "--domain",
        type=str,
        help="The domain that the rule needs to be disabled for",
    )
    parser.add_argument("-r", "--rule", type=int, help="Rule to be disabled")
    parser.add_argument(
        "-o",
        "--off",
        action="store_true",
        help="Used to disable all rules for the domain",
    )
    parser.add_argument(
        "-R",
        "--reset",
        action="store_true",
        help="Wipe the modsec file, essentially re-enabling all rules.")
    args = parser.parse_args()

    if not args.user or not args.domain or not (args.off or args.rule or args.reset):
        parser.print_help()
        exit(1)

    disable_rule(args)
    rebuild_restart_apache()


def disable_rule(args):
    """Disables the rule provided or disables all rules for the domain."""
    apache_dir = Path("/usr/local/apache/conf/userdata/std/2")
    user_dir = apache_dir / args.user / args.domain
    user_dir.mkdir(parents=True, exist_ok=True)

    modsec_file = user_dir / "modsec.conf"
    modsec_file.touch()

    if args.reset:
        modsec_file.write_text("")
        return # Empty the file and return so we can rebuild/restart.

    lines = []
    if args.off:
        lines.append("SecRuleRemoveById Off")
    else:
        lines = []
        with modsec_file.open("r") as stream:
            for line in stream.readlines():
                line = line.strip()
                if line != "SecRuleRemoveById Off" and line != "":
                    lines.append(line)

        new_rule = f"SecRuleRemoveById {args.rule}"
        if new_rule in lines:
            print(red(f"Rule {args.rule} is already disabled for this domain!"))
            exit(1)

        lines.append(new_rule)

    print(green(f"Writing to file: {modsec_file}"))
    with modsec_file.open("w") as out:
        for line in lines:
            out.write(line + "\n")

def rebuild_restart_apache():
    """As the name suggests, rebuilds httpd conf and then gracefully restarts
    Apache.
    """
    subprocess.run(["/scripts/rebuildhttpdconf"])
    subprocess.run(["/sbin/apachectl", "graceful"])


if __name__ == "__main__":
    main()
